import React, { useRef } from 'react';

const hasOwn = Object.prototype.hasOwnProperty;

function is(x, y) {
  if (x === y) {
    return x !== 0 || y !== 0 || 1 / x === 1 / y;
  }

  return x !== x && y !== y;
}

function shallowEqual(objA, objB) {
  if (is(objA, objB)) return true;

  if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
    return false;
  }

  const keysA = Object.keys(objA);
  const keysB = Object.keys(objB);
  if (keysA.length !== keysB.length) return false;

  for (let i = 0; i < keysA.length; i++) {
    if (!hasOwn.call(objB, keysA[i]) || !is(objA[keysA[i]], objB[keysA[i]])) {
      return false;
    }
  }

  return true;
}

function shallowEquals(A, B) {
  if (!Array.isArray(A) || !Array.isArray(B)) return false;

  if (A.some((item, i) => !shallowEqual(item, B[i]))) {
    return false;
  }

  return true;
}

function useShallowDeps(lastestDeps) {
  const depsRef = useRef(lastestDeps);

  if (!shallowEquals(depsRef.current, lastestDeps)) {
    depsRef.current = lastestDeps;
  }

  return depsRef.current;
}

const isPrimitive = val => val !== Object(val);

const deprecatedPrimitiveDeps = (lastestDeps, hookName) => {
  const originName = `use${hookName.slice(10)}`;

  if (!lastestDeps || !lastestDeps.length) {
    return console.warn(`\`${hookName}\` should not be used with no dependencies. Use React.${originName} instead.`);
  }

  if (lastestDeps.every(isPrimitive)) {
    return console.warn(`\`${hookName}\` should not be used with dependencies that are all primitive values. Use React.${originName} instead.`);
  }
};

const useShallowEffect = function (cb, deps) {
  if (process.env.NODE_ENV !== 'production') {
    deprecatedPrimitiveDeps(deps, 'useShallowEffect');
  }

  return React.useEffect(cb, useShallowDeps(deps));
};

const useShallowLayoutEffect = function (cb, deps) {
  if (process.env.NODE_ENV !== 'production') {
    deprecatedPrimitiveDeps(deps, 'useShallowLayoutEffect');
  }

  return React.useLayoutEffect(cb, useShallowDeps(deps));
};

const useShallowMemo = function (cb, deps) {
  if (process.env.NODE_ENV !== 'production') {
    deprecatedPrimitiveDeps(deps, 'useShallowMemo');
  }

  return React.useMemo(cb, useShallowDeps(deps));
};

const useShallowCallback = function (cb, deps) {
  if (process.env.NODE_ENV !== 'production') {
    deprecatedPrimitiveDeps(deps, 'useShallowCallback');
  }

  return React.useCallback(cb, useShallowDeps(deps));
};

const useShallowImperativeHandle = function (ref, cb, deps) {
  if (process.env.NODE_ENV !== 'production') {
    deprecatedPrimitiveDeps(deps, 'useShallowImperativeHandle');
  }

  return React.useImperativeHandle(ref, cb, useShallowDeps(deps));
};

export { useShallowCallback, useShallowEffect, useShallowImperativeHandle, useShallowLayoutEffect, useShallowMemo };
